{===============================================================================
  Unit QRPreviewToolBar

  This unit contains the TQRPreviewToolBar component which is a toolbar that
  can be used in custom QuickReport previews.
  Tested with Delphi 3 and QuickReport 2.

  Author:      Craig Manley
  Created:     24 November 1999
  Last update: 26 November 1999
===============================================================================}


unit QRPreviewToolBar;
{$INCLUDE DFS.INC}

{$IFDEF DFS_DELPHI_4_UP}
 {$DEFINE QR3}
{$ELSE}
 {$DEFINE QR2}
{$ENDIF}

interface

uses
  Windows, Messages, Classes, Controls, ExtCtrls, Buttons,
  forms, ComCtrls, Spin, stdctrls, dialogs, SysUtils,
  qrptcnst,{$IFDEF QR2}qr2const{$ELSE}qr3const{$ENDIF},qrprntr,quickrpt;

type
  TQRPreviewTool = (qrptLabelPage,qrptSpinEditPage,
                    qrptBtnFirst,qrptBtnPrev,qrptBtnNext,qrptBtnLast,
                    qrptDiv1,
                    qrptLabelZoom,qrptSpinEditZoom,
                    qrptBtnZoomToFit,qrptBtnZoomTo100,qrptBtnZoomToWidth,
                    qrptDiv2,
                    qrptBtnPrintSetup,
                    qrptBtnPrint,
                    qrptDiv3,
                    qrptBtnOpen,
                    qrptBtnSave,
                    qrptDiv4,
                    qrptBtnClose);

  TQRPreviewTools = set of TQRPreviewTool;

  TQRPreviewOption = (qrpoPageEdit,
                      qrpoNavButtons,
                      qrpoZoomEdit,
                      qrpoZoomButtons,
                      qrpoPrintSetupButton,
                      qrpoPrintButton,
                      qrpoOpenButton,
                      qrpoSaveButton,
                      qrpoCloseButton);

  TQRPreviewOptions = set of TQRPreviewOption;

const
  QRPT_PAGEEDIT       = 1 shl Ord(qrptLabelPage) +
                        1 shl Ord(qrptSpinEditPage);
                        { 00000000 00000000 00000011 } { 1,2 }
  QRPT_NAVBUTTONS     = 1 shl Ord(qrptBtnFirst) +
                        1 shl Ord(qrptBtnPrev) +
                        1 shl Ord(qrptBtnNext) +
                        1 shl Ord(qrptBtnLast);
                        { 00000000 00000000 00111100 } { 4,8,$10,$20 }
  QRPT_ZOOMEDIT       = 1 shl Ord(qrptLabelZoom) +
                        1 shl Ord(qrptSpinEditZoom);
                        { 00000000 00000001 10000000 } { $80,$100 }
  QRPT_ZOOMBUTTONS    = 1 shl Ord(qrptBtnZoomToFit) +
                        1 shl Ord(qrptBtnZoomTo100) +
                        1 shl Ord(qrptBtnZoomToWidth);
                        { 00000000 00001110 00000000 } { $200,$400,$800 }
  QRPT_PRINTSETUPBTN  = 1 shl Ord(qrptBtnPrintSetup);
                        { 00000000 00010000 00000000 } { $1000 }
  QRPT_PRINTBUTTON    = 1 shl Ord(qrptBtnPrint);
                        { 00000000 00100000 00000000 } { $2000 }
  QRPT_OPENBUTTON     = 1 shl Ord(qrptBtnOpen);
                        { 00000000 01000000 00000000 } { $4000 }
  QRPT_SAVEBUTTON     = 1 shl Ord(qrptBtnSave);
                        { 00000000 10000000 00000000 } { $8000 }
  QRPT_CLOSEBUTTON   =  1 shl Ord(qrptBtnClose);
                        { 00000001 00000000 00000000 } { $10000 }

  QRPreviewOptionMask: array[TQRPreviewOption] of DWORD = (QRPT_PAGEEDIT,
                                                           QRPT_NAVBUTTONS,
                                                           QRPT_ZOOMEDIT,
                                                           QRPT_ZOOMBUTTONS,
                                                           QRPT_PRINTSETUPBTN,
                                                           QRPT_PRINTBUTTON,
                                                           QRPT_OPENBUTTON,
                                                           QRPT_SAVEBUTTON,
                                                           QRPT_CLOSEBUTTON);

type
  TQRPreviewToolGlyphRes = record
    ID: TQRPreviewTool;
    sRes: PChar;
  end;

const
  BitMapResources: array [0..11] of TQRPreviewToolGlyphRes = (
                   (ID:qrptBtnFirst      ;sRes:'QRFIRSTPAGEBITMAP'),
                   (ID:qrptBtnPrev       ;sRes:'QRPREVPAGEBITMAP'),
                   (ID:qrptBtnNext       ;sRes:'QRNEXTPAGEBITMAP'),
                   (ID:qrptBtnLast       ;sRes:'QRLASTPAGEBITMAP'),
                   (ID:qrptBtnZoomToFit  ;sRes:'QRZOOMTOFITBITMAP'),
                   (ID:qrptBtnZoomTo100  ;sRes:'QRZOOMTO100BITMAP'),
                   (ID:qrptBtnZoomToWidth;sRes:'QRZOOMTOWIDTHBITMAP'),
                   (ID:qrptBtnPrintSetup ;sRes:'QRPRINTSETUPBITMAP'),
                   (ID:qrptBtnPrint      ;sRes:'QRPRINTBITMAP'),
                   (ID:qrptBtnOpen       ;sRes:'QROPENBITMAP'),
                   (ID:qrptBtnSave       ;sRes:'QRSAVEBITMAP'),
                   (ID:qrptBtnClose      ;sRes:'BBCLOSE'));

type
  TQRLanguage = (qrlgEnglish,qrlgDutch);

  TQRPTBNotifyEvent = procedure (Sender: TObject; var bHandled: boolean) of object;

  TQRPreviewToolBar = class(TCustomPanel)
  private
    FLanguage: TQRLanguage;
    FOnBeforePrintSetup:   TQRPTBNotifyEvent;
    FOnBeforePrint:        TQRPTBNotifyEvent;
    FOnBeforeOpen:         TQRPTBNotifyEvent;
    FOnSave:         TNotifyEvent;
    FQRPreview: TQRPreview;
    FPreviewOptions: TQRPreviewOptions;
    FPrintOnSetupOk: boolean;
    FQuickRep: TQuickRep;
    FShowHint: boolean;
    FStrPage,
    FStrPages,
    FStrFiles,
    FStrFailLoadFile: string;
    FToolBarItems: array[TQRPreviewTool] of TControl;
    procedure ReAlignControls;
    procedure InsertToolBarItem(ControlClass: TControlClass; QRPreviewTool: TQRPreviewTool);
    procedure SetLanguage(Value: TQRLanguage);
    procedure SetPreviewOptions(Value: TQRPreviewOptions);
    procedure SetShowHint(Value: boolean);
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
  protected
    procedure DoEvent(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Reset;
    procedure ResetPageNumber;
    procedure SimulateZoomTo100;
    procedure SimulateZoomToFit;
    procedure SimulateZoomToWidth;
  published
    property Align;
    property BevelInner;
    property BevelOuter;
    property BorderStyle;
    property BevelWidth;
    property Color;
    property Ctl3D;
    property Cursor;
    property Enabled;
    property Height;
    property Language: TQRLanguage read FLanguage write SetLanguage;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property PopupMenu;
    property OnBeforePrintSetup: TQRPTBNotifyEvent read FOnBeforePrintSetup write FOnBeforePrintSetup;
    property OnBeforePrint:      TQRPTBNotifyEvent read FOnBeforePrint write FOnBeforePrint;
    property OnBeforeOpen:       TQRPTBNotifyEvent read FOnBeforeOpen write FOnBeforeOpen;
    property OnSave: TNotifyEvent read FOnSave write FOnSave;
    property PreviewOptions: TQRPreviewOptions read FPreviewOptions write SetPreviewOptions;
    property PrintOnSetupOk: boolean read FPrintOnSetupOk write FPrintOnSetupOk;
    property QuickRep: TQuickRep read FQuickRep write FQuickRep;
    property QRPreview: TQRPreview read FQRPreview write FQRPreview;
    property ShowHint: boolean read FShowHint write SetShowHint;
    property Width;
    property Visible;
  end;

  TDividerSpeedButton = class(TSpeedButton)
  public
    constructor Create(AOwner: TComponent); override;
  end;

procedure Register;

implementation

{$R *.RES}

procedure Register;
begin
  RegisterComponents('Freeware', [TQRPreviewToolBar]);
  RegisterComponents('Freeware', [TDividerSpeedButton]);
end;


function GetCompFromOwnerHierarchy(AComponent: TComponent; AClassType: TClass): TComponent;
var
  Owner: TComponent;
begin
  Result := nil;
  if AComponent = nil then
    Exit;
  Owner := AComponent.Owner;
  while Owner <> nil do
  begin
    if Owner is AClassType then
    begin
      Result := Owner;
      Exit;
    end;
    Owner := Owner.Owner;
  end;
end;


{=========================== TDividerSpeedButton ==============================}
constructor TDividerSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Enabled := False;
  Flat := True;
  Width := 8;
  {$IFDEF DFS_DELPHI_4_UP}
  Transparent := True;
  {$ENDIF}
  Glyph.Handle := LoadBitmap(hInstance,PChar(AnsiUpperCase(ClassName)));
  NumGlyphs := 2;
end;
{==============================================================================}


{============================= TQRPreviewToolBar ==============================}
{------------------------------- Public methods -------------------------------}
constructor TQRPreviewToolBar.Create(AOwner: TComponent);
var
  x: TQRPreviewTool;
  i: integer;
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csSetCaption];
  Align := alTop;
  BevelOuter := bvNone;
  Height := 25;
  FPrintOnSetupOk := True;
  InsertToolBarItem(TLabel,qrptLabelPage);            // Page
  InsertToolBarItem(TSpinEdit,qrptSpinEditPage);
  InsertToolBarItem(TSpeedButton,qrptBtnFirst);       // First
  InsertToolBarItem(TSpeedButton,qrptBtnPrev);        // Previous
  InsertToolBarItem(TSpeedButton,qrptBtnNext);        // Next
  InsertToolBarItem(TSpeedButton,qrptBtnLast);        // Last
  InsertToolBarItem(TDividerSpeedButton,qrptDiv1);
  InsertToolBarItem(TLabel,qrptLabelZoom);            // Zoom
  InsertToolBarItem(TSpinEdit,qrptSpinEditZoom);
  InsertToolBarItem(TSpeedButton,qrptBtnZoomToFit);   // Zoom to fit
  InsertToolBarItem(TSpeedButton,qrptBtnZoomTo100);   // Zoom to 100%
  InsertToolBarItem(TSpeedButton,qrptBtnZoomToWidth); // Zoom to width
  InsertToolBarItem(TDividerSpeedButton,qrptDiv2);
  InsertToolBarItem(TSpeedButton,qrptBtnPrintSetup); // Print setup
  InsertToolBarItem(TSpeedButton,qrptBtnPrint);      // Print
  InsertToolBarItem(TDividerSpeedButton,qrptDiv3);
  InsertToolBarItem(TSpeedButton,qrptBtnOpen);       // Load
  InsertToolBarItem(TSpeedButton,qrptBtnSave);       // Save
  InsertToolBarItem(TDividerSpeedButton,qrptDiv4);
  InsertToolBarItem(TSpeedButton,qrptBtnClose);     // Close
  // Set QRPreview property
  if AOwner is TCustomForm then
  begin
    for i := 0 to TCustomForm(AOwner).ComponentCount-1 do
    begin
      if TCustomForm(AOwner).Components[i] is TQRPreview then
      begin
        FQRPreview := TQRPreview(TCustomForm(AOwner).Components[i]);
      end;
    end;
  end;
  // Set SpinEdit limits
  TspinEdit(FToolBarItems[qrptSpinEditPage]).MaxValue := 9999;
  TSpinEdit(FToolBarItems[qrptSpinEditPage]).MinValue := 1;
  TSpinEdit(FToolBarItems[qrptSpinEditZoom]).MaxValue := 300;
  TSpinEdit(FToolBarItems[qrptSpinEditZoom]).MinValue := 30;
  TSpinEdit(FToolBarItems[qrptSpinEditPage]).Value := 1;
  for x := Low(FToolBarItems) to High(FToolBarItems) do
  begin
    // Set the TLabel.FocusControl to the adjacent TSpinEdits
    if (FToolBarItems[x] is TLabel) and (FToolBarItems[TQRPreviewTool(Ord(x)+1)] is TSpinEdit) then
      TLabel(FToolBarItems[x]).FocusControl := TSpinEdit(FToolBarItems[TQRPreviewTool(Ord(x)+1)]);
    // Set click event handlers
    if (FToolBarItems[x].ClassType = TSpeedButton) then
      TSpeedButton(FToolBarItems[x]).OnClick := DoEvent
    // Set OnChange event handlers
    else if (FToolBarItems[x] is TSpinEdit) then
      TSpinEdit(FToolBarItems[x]).OnChange := DoEvent
    // Set vertical text alignment in TLabels
    else if (FToolBarItems[x] is TLabel) then
      TLabel(FToolBarItems[x]).Layout := tlCenter;
    // Load glyphs
    for i := Low(BitMapResources) to High(BitMapResources) do
    begin
      if BitMapResources[i].ID = x then
      begin
        TSpeedButton(FToolBarItems[x]).Glyph.Handle := LoadBitmap(hInstance,BitMapResources[i].sRes);
        Break;
      end;
    end;
  end;
  // Set the group index for the zoom buttons
  for x := qrptBtnZoomToFit to qrptBtnZoomToWidth do
    TSpeedButton(FToolBarItems[x]).GroupIndex := 1;
  // Set language
  SetLanguage(qrlgEnglish);
  // Show hints
  SetShowHint(True);
  // Show all items
  SetPreviewOptions([qrpoPageEdit,
                     qrpoNavButtons,
                     qrpoZoomEdit,
                     qrpoZoomButtons,
                     qrpoPrintSetupButton,
                     qrpoPrintButton,
                     qrpoOpenButton,
                     qrpoSaveButton,
                     qrpoCloseButton]);
end;

destructor TQRPreviewToolBar.Destroy;
var
  i: integer;
begin
  for i := ControlCount-1 downto 0 do
    Controls[i].Free;
  inherited Destroy;
end;

procedure TQRPreviewToolBar.ResetPageNumber;
var
  pEvent: TNotifyEvent;
begin
  if (FQRPreview <> nil) then
  begin
    // qrptSpinEditPage
    if (FQRPreview.QRPrinter <> nil) then
    begin
      pEvent := TSpinEdit(FToolBarItems[qrptSpinEditPage]).OnChange;
      TSpinEdit(FToolBarItems[qrptSpinEditPage]).OnChange := nil;
      TSpinEdit(FToolBarItems[qrptSpinEditPage]).Value := FQRPreview.PageNumber;
      TSpinEdit(FToolBarItems[qrptSpinEditPage]).OnChange := pEvent;
    end;
  end;
end;

procedure TQRPreviewToolBar.Reset;
var
  pEvent: TNotifyEvent;
begin
  ResetPageNumber;
  if (FQRPreview <> nil) then
  begin
    // qrptSpinEditZoom
    pEvent := TSpinEdit(FToolBarItems[qrptSpinEditZoom]).OnChange;
    TSpinEdit(FToolBarItems[qrptSpinEditZoom]).OnChange := nil;
    TSpinEdit(FToolBarItems[qrptSpinEditZoom]).Value := FQRPreview.Zoom;
    TSpinEdit(FToolBarItems[qrptSpinEditZoom]).OnChange := pEvent;
  end;
  SimulateZoomToWidth;
end;

procedure TQRPreviewToolBar.SimulateZoomTo100;
begin
  with TSpeedButton(FToolBarItems[qrptBtnZoomTo100]) do
  begin
    Click;
    Down := True;
  end;
end;

procedure TQRPreviewToolBar.SimulateZoomToFit;
begin
  with TSpeedButton(FToolBarItems[qrptBtnZoomToFit]) do
  begin
    Click;
    Down := True;
  end;
end;

procedure TQRPreviewToolBar.SimulateZoomToWidth;
begin
  with TSpeedButton(FToolBarItems[qrptBtnZoomToWidth]) do
  begin
    Click;
    Down := True;
  end;
end;

{------------------------------- Private methods ------------------------------}
procedure TQRPreviewToolBar.InsertToolBarItem(ControlClass: TControlClass; QRPreviewTool: TQRPreviewTool);
begin
  FToolBarItems[QRPreviewTool] := ControlClass.Create(self);
  InsertControl(FToolBarItems[QRPreviewTool]);
  FToolBarItems[QRPreviewTool].Tag := Ord(QRPreviewTool);
  FToolBarItems[QRPreviewTool].Height := 25;
  if FToolBarItems[QRPreviewTool] is TSpinEdit then
    TSpinEdit(FToolBarItems[QRPreviewTool]).Width := 49;
  FToolBarItems[QRPreviewTool].Visible := True;
end;

procedure TQRPreviewToolBar.ReAlignControls;
  {----------------------------------------------------}
  function CaptionToStr(const sCaption: string): string;
  var
    lviAmpCnt, i,j: integer;
  begin
    lviAmpCnt := 0;
    Result := '';
    for i := 1 to Length(sCaption) do
    begin
      if sCaption[i] = '&' then
        inc(lviAmpCnt)
      else
      begin
        for j := 1 to (lviAmpCnt shr 1) do
          Result := Result + '&';
        lviAmpCnt := 0;
        Result := Result + sCaption[i];
      end;
    end;
  end;
  {----------------------------------------------------}
var
  x: TQRPreviewTool;
  lviLastVisibleTool: integer;
begin
  lviLastVisibleTool := -1;
  for x := Low(FToolBarItems) to High(FToolBarItems) do
  begin
    if lviLastVisibleTool = -1 then
      FToolBarItems[x].Left := 0
    else
      FToolBarItems[x].Left := FToolBarItems[TQRPreviewTool(lviLastVisibleTool)].Left + FToolBarItems[TQRPreviewTool(lviLastVisibleTool)].Width;
    if FToolBarItems[x].Visible then
      lviLastVisibleTool := Ord(x);
    if (FToolBarItems[x] is TSpeedButton) and not (FToolBarItems[x] is TDividerSpeedButton) then
    begin
      if TSpeedButton(FToolBarItems[x]).Glyph.Width > 25 then
        TSpeedButton(FToolBarItems[x]).NumGlyphs := 2;
      if TSpeedButton(FToolBarItems[x]).Caption = '' then
        TSpeedButton(FToolBarItems[x]).Width := 25
      else
      begin
        if (Parent <> nil) then
          FToolBarItems[x].Width := FToolBarItems[x].Width - FToolBarItems[x].ClientWidth +
                                    35 +
                                    Canvas.TextWidth(CaptionToStr(TSpeedButton(FToolBarItems[x]).Caption));
      end;
    end
    else if (FToolBarItems[x] is TLabel) then
      TLabel(FToolBarItems[x]).Height := 25;
  end;
end;

procedure TQRPreviewToolBar.SetLanguage(Value: TQRLanguage);
begin
  FLanguage := Value;
  case FLanguage of
    qrlgDutch:
    begin
      // Load strings for hints
      FToolBarItems[qrptLabelPage].Hint      := SNLHintPage;
      FToolBarItems[qrptSpinEditPage].Hint   := SNLHintPage;
      FToolBarItems[qrptBtnFirst].Hint       := SNLHintFirstPage;
      FToolBarItems[qrptBtnPrev].Hint        := SNLHintPrevPage;
      FToolBarItems[qrptBtnNext].Hint        := SNLHintNextPage;
      FToolBarItems[qrptBtnLast].Hint        := SNLHintLastPage;

      FToolBarItems[qrptLabelZoom].Hint      := SNLHintZoom;
      FToolBarItems[qrptSpinEditZoom].Hint   := SNLHintZoom;
      FToolBarItems[qrptBtnZoomToFit].Hint   := SNLHintZoomToFit;
      FToolBarItems[qrptBtnZoomTo100].Hint   := SNLHintZoom100;
      FToolBarItems[qrptBtnZoomToWidth].Hint := SNLHintZoomToWidth;

      FToolBarItems[qrptBtnPrintSetup].Hint  := SNLHintPrinterSetup;
      FToolBarItems[qrptBtnPrint].Hint       := SNLHintPrintReport;

      FToolBarItems[qrptBtnOpen].Hint        := SNLHintOpenReport;
      FToolBarItems[qrptBtnSave].Hint        := SNLHintSaveReport;

      FToolBarItems[qrptBtnClose].Hint       := SNLHintClose;
      // Load captions
      TLabel(FToolBarItems[qrptLabelPage]).Caption      := SNLCaptionPage;
      TLabel(FToolBarItems[qrptLabelZoom]).Caption      := SNLCaptionZoom;
      TSpeedButton(FToolBarItems[qrptBtnClose]).Caption := SNLCaptionClose;
      // Load strings
      FStrPage         := SNLPage;
      FStrPages        := SNLPages;
      FStrFiles        := SNLFiles;
      FStrFailLoadFile := SNLFailLoadFile;
    end;
    qrlgEnglish:
    begin
      // Load strings for hints
      FToolBarItems[qrptLabelPage].Hint      := SENHintPage;
      FToolBarItems[qrptSpinEditPage].Hint   := SENHintPage;
     {$IFDEF QR2}
      FToolBarItems[qrptBtnFirst].Hint       := LoadStr( SqrFirstPage );
      FToolBarItems[qrptBtnPrev].Hint        := LoadStr( SqrPrevPage );
      FToolBarItems[qrptBtnNext].Hint        := LoadStr( SqrNextPage );
      FToolBarItems[qrptBtnLast].Hint        := LoadStr( SqrLastPage );
     {$ELSE}
      FToolBarItems[qrptBtnFirst].Hint       := SqrFirstPage;
      FToolBarItems[qrptBtnPrev].Hint        := SqrPrevPage;
      FToolBarItems[qrptBtnNext].Hint        := SqrNextPage;
      FToolBarItems[qrptBtnLast].Hint        := SqrLastPage;
     {$ENDIF}
      FToolBarItems[qrptLabelZoom].Hint      := SENHintZoom;
      FToolBarItems[qrptSpinEditZoom].Hint   := SENHintZoom;
     {$IFDEF QR2}
      FToolBarItems[qrptBtnZoomToFit].Hint   := LoadStr( SqrZoomToFit );
      FToolBarItems[qrptBtnZoomTo100].Hint   := LoadStr( SqrZoom100 );
      FToolBarItems[qrptBtnZoomToWidth].Hint := LoadStr( SqrZoomToWidth );
     {$ELSE}
      FToolBarItems[qrptBtnZoomToFit].Hint   := SqrZoomToFit;
      FToolBarItems[qrptBtnZoomTo100].Hint   := SqrZoom100;
      FToolBarItems[qrptBtnZoomToWidth].Hint := SqrZoomToWidth;
     {$ENDIF}
     {$IFDEF QR2}
      FToolBarItems[qrptBtnPrintSetup].Hint  := LoadStr( SqrPrinterSetup );
      FToolBarItems[qrptBtnPrint].Hint       := LoadStr( SqrPrintReport );

      FToolBarItems[qrptBtnOpen].Hint        := LoadStr( SqrSaveReport );
      FToolBarItems[qrptBtnSave].Hint        := LoadStr( SqrLoadReport );

      FToolBarItems[qrptBtnClose].Hint       := LoadStr( SqrCancel );
     {$ELSE}
      FToolBarItems[qrptBtnPrintSetup].Hint  := SqrPrinterSetup;
      FToolBarItems[qrptBtnPrint].Hint       := SqrPrintReport;

      FToolBarItems[qrptBtnOpen].Hint        := SqrSaveReport;
      FToolBarItems[qrptBtnSave].Hint        := SqrLoadReport;

      FToolBarItems[qrptBtnClose].Hint       := SqrCancel;
     {$ENDIF}
      // Load captions
      TLabel(FToolBarItems[qrptLabelPage]).Caption      := SENCaptionPage;
      TLabel(FToolBarItems[qrptLabelZoom]).Caption      := SENCaptionZoom;
      TSpeedButton(FToolBarItems[qrptBtnClose]).Caption := FToolBarItems[qrptBtnClose].Hint;
      // Load strings
      FStrPage         := SENPage;
      FStrPages        := SENPages;
      FStrFiles        := SENFiles;
      FStrFailLoadFile := SENFailLoadFile;
    end;
  end;
  ReAlignControls;
end;

procedure TQRPreviewToolBar.SetPreviewOptions(Value: TQRPreviewOptions);
var
  o: TQRPreviewOption;
  t: TQRPreviewTool;
  lvbSet: boolean;
  lviLastVisibleTool: integer;
begin
  for o := Low(QRPreviewOptionMask) to High(QRPreviewOptionMask) do
  begin
    lvbSet := (o in Value);
    for t := Low(FToolBarItems) to High(FToolBarItems) do
    begin
      if ((1 shl Ord(t)) and QRPreviewOptionMask[o]) > 0 then
      begin
        FToolBarItems[t].Enabled := lvbSet;
        FToolBarItems[t].Visible := lvbSet;
      end;
    end;
  end;
  FToolBarItems[qrptDiv1].Visible := (qrpoZoomEdit in Value) or (qrpoZoomButtons in Value);
  FToolBarItems[qrptDiv2].Visible := (qrpoPrintSetupButton in Value) or (qrpoPrintButton in Value);
  FToolBarItems[qrptDiv3].Visible := (qrpoOpenButton in Value) or (qrpoSaveButton in Value);
  FToolBarItems[qrptDiv4].Visible := (qrpoCloseButton in Value);
  // Make sure that all visible TDividerSpeedButtons are seperated.
  lviLastVisibleTool := -1;
  for t := Low(FToolBarItems) to High(FToolBarItems) do
  begin
    if (FToolBarItems[t] is TDividerSpeedButton) then
    begin
      if lviLastVisibleTool >= 0 then
      begin
        if FToolBarItems[TQRPreviewTool(lviLastVisibleTool)] is TDividerSpeedButton then
          TDividerSpeedButton(FToolBarItems[TQRPreviewTool(lviLastVisibleTool)]).Visible := False;
      end;
    end;
    if FToolBarItems[t].Visible then
      lviLastVisibleTool := Ord(t);
  end;
  FPreviewOptions := Value;
  ReAlignControls;
end;

procedure TQRPreviewToolBar.SetShowHint(Value: boolean);
var
  x: TQRPreviewTool;
begin
  for x := Low(FToolBarItems) to High(FToolBarItems) do
    TControl(FToolBarItems[x]).ShowHint := Value;
  FShowHint := Value;
end;

procedure TQRPreviewToolBar.WMCreate(var Message: TWMCreate);
begin
  ReAlignControls;
  inherited;
end;

{------------------------------- Protected methods ----------------------------}
procedure TQRPreviewToolBar.DoEvent(Sender: TObject);
var
  lvbHandled: boolean;
  frm: TCustomForm;
begin
  if FQRPreview <> nil then
  begin
    lvbHandled := False;
    case TComponent(Sender).Tag of
      Ord(qrptSpinEditPage):          // OnPageChange
      begin
        if (FQRPreview.QRPrinter <> nil) then
        begin
          if (TSpinEdit(Sender).Value > FQRPreview.QRPrinter.PageCount) then
            TSpinEdit(Sender).Value := FQRPreview.QRPrinter.PageCount
          else if (TSpinEdit(Sender).Value <= 0) then
            TSpinEdit(Sender).Value := Ord(FQRPreview.QRPrinter.PageCount > 0);
          if TSpinEdit(Sender).Value > 0 then
            FQRPreview.PageNumber := TSpinEdit(Sender).Value;
        end;
      end;
      Ord(qrptSpinEditZoom):          // OnZoomChange
        FQRPreview.Zoom := TSpinEdit(Sender).Value;
      Ord(qrptBtnFirst):              // OnFirst
      begin
        if (FQRPreview.QRPrinter <> nil) then
          FQRPreview.PageNumber := 1;
      end;
      Ord(qrptBtnPrev):               // OnPrevious
      begin
        if (FQRPreview.PageNumber > 1) then
        begin
          if (FQRPreview.QRPrinter <> nil) then
            FQRPreview.PageNumber := FQRPreview.PageNumber - 1;
        end;
      end;
      Ord(qrptBtnNext):               // OnNext
      begin
        if (FQRPreview.PageNumber < FQRPreview.QRPrinter.PageCount) then
        begin
          if (FQRPreview.QRPrinter <> nil) then
            FQRPreview.PageNumber := FQRPreview.PageNumber + 1;
        end;
      end;
      Ord(qrptBtnLast):               // OnLast
      begin
        if (FQRPreview.QRPrinter <> nil) then
          FQRPreview.PageNumber := FQRPreview.QRPrinter.PageCount;
      end;
      Ord(qrptBtnZoomToFit):          // OnZoomToFit
      begin
        FQRPreview.ZoomToFit;
        TSpinEdit(FToolBarItems[qrptSpinEditZoom]).Value := FQRPreview.Zoom;
      end;
      Ord(qrptBtnZoomTo100):          // OnZoomTo100
      begin
        FQRPreview.Zoom := 100;
        TSpinEdit(FToolBarItems[qrptSpinEditZoom]).Value := FQRPreview.Zoom;
      end;
      Ord(qrptBtnZoomToWidth):        // OnZoomToWidth
      begin
        FQRPreview.ZoomToWidth;
        TSpinEdit(FToolBarItems[qrptSpinEditZoom]).Value := FQRPreview.Zoom;
      end;
      Ord(qrptBtnPrintSetup):         // OnBeforePrintSetup
      begin
        if Assigned(FOnBeforePrintSetup) then
          FOnBeforePrintSetup(self,lvbHandled);
        if (not lvbHandled) and (FQRPreview.QRPrinter <> nil) then
        begin
        { With 2.0j, QuickReport will set the report's tag property to -1 if the
          user cancels the printersetup.  By checking for this value, we can call
          the print method directly from setup if the users selects OK }
          if FQuickRep <> nil then
            FQuickRep.Tag := -1; { Just in case you are using an older version }
          FQRPreview.QRPrinter.PrintSetup;
          if FPrintOnSetupOk and (FQuickRep <> nil) and (FQuickRep.Tag = 0) then
            TSpeedButton(FToolBarItems[qrptBtnPrint]).Click;
        end;
      end;
      Ord(qrptBtnPrint):              // OnBeforePrint
      begin
        if Assigned(FOnBeforePrint) then
          FOnBeforePrint(self,lvbHandled);
        if (not lvbHandled) and (FQRPreview.QRPrinter <> nil) then
        begin
          TSpeedButton(Sender).Enabled := False;
          try
            FQRPreview.QRPrinter.Print;
          finally
            TSpeedButton(Sender).Enabled := True;
          end;
        end;
      end;
      Ord(qrptBtnOpen):               // OnBeforeOpen
      begin
        if Assigned(FOnBeforeOpen) then
          FOnBeforeOpen(self,lvbHandled);
        if (not lvbHandled) and (FQRPreview.QRPrinter <> nil) then
        with TOpenDialog.Create(self) do
        try
          Filter := 'QuickReport ' + FStrFiles + ' (*.qrp)|*.qrp';
          if Execute then
          begin
            if FileExists(FileName) then
            begin
              FQRPreview.QRPrinter.Load(Filename);
              FQRPreview.PageNumber := 1;
              FQRPreview.PreviewImage.PageNumber := 1;
              if QRPreview.QRPrinter.Title = '' then
                QRPreview.QRPrinter.Title := Filename;
            end
            else
              MessageBoxA(self.Handle,PChar(FStrFailLoadFile + ': ' + FileName),'Error',MB_ICONERROR);
          end;
        finally
          Free;
        end;
      end;
      Ord(qrptBtnSave):               // OnSave
      begin
        if Assigned(FOnBeforeOpen) then
          FOnSave(self);
      end;
    end; { case }
    if (TComponent(Sender).Tag >= Ord(qrptBtnZoomToFit)) and (TComponent(Sender).Tag <= Ord(qrptBtnZoomToWidth)) then
      TSpinEdit(FToolBarItems[qrptSpinEditZoom]).Value := FQRPreview.Zoom
    else if (TComponent(Sender).Tag >= Ord(qrptBtnFirst)) and (TComponent(Sender).Tag <= Ord(qrptBtnLast)) then
      TSpinEdit(FToolBarItems[qrptSpinEditPage]).Value := FQRPreview.PageNumber;
  end;
  // OnClose
  if TComponent(Sender).Tag = Ord(qrptBtnClose) then
  begin
    frm := TCustomForm(GetCompFromOwnerHierarchy(self,TCustomForm));
    if (frm <> nil) then
      TCustomForm(frm).Close;
  end;
end;


end.
